@@ -114,6 +114,9 @@ void remove_suspend_item();
 static void suspend_toggle_cb(MBMenuItem *item);
 static void screensaver_toggle_cb(MBMenuItem *item);
 void backlight_timeout_callback ( MBTrayApp *app );
+static gboolean key_event_callback( GtkWidget      *widget, 
+				    GdkEventKey    *event,
+				    gpointer        data );
 
 #ifdef USE_LIBSN
 static void sn_exec(char* name, char* bin_name, char *desc);
@@ -130,6 +133,11 @@ static void sn_exec(char* name, char* bin_name, char *desc);
 #define UP   0
 #define DOWN 1
 
+#define KEY_JOG_DOWN	65364
+#define KEY_JOG_UP	65362
+#define KEY_OK		65293
+#define KEY_CANCEL	65307
+
 #define MBMAX(x,y) ((x>y)?(x):(y))
 
 #define NEW(OBJ) ((OBJ *)(malloc(sizeof(OBJ))))
@@ -414,6 +422,7 @@ blanker_cb(void)
 
   gtk_widget_show_all (blanker_window);
   gdk_pointer_grab (blanker_window->window, TRUE, GDK_BUTTON_PRESS_MASK, NULL, NULL, CurrentTime);
+  gdk_keyboard_grab (blanker_window->window, TRUE, CurrentTime);
   PopupIsMapped = True;
 }
 
@@ -642,6 +651,47 @@ xevent_callback (MBTrayApp *app, XEvent *ev)
     }
 }
 
+static gboolean key_event_callback( GtkWidget      *widget, 
+				    GdkEventKey	   *event,
+				    gpointer        data )
+{
+  if (event->type == GDK_KEY_PRESS) {
+    //printf("key pressed (%d)\n", event->keyval);
+  } else if (event->type == GDK_KEY_RELEASE) {
+    //printf("key released (%d)\n", event->keyval);
+
+    /* Return if xmms isn't running */
+    if (WEXITSTATUS(system("pidof -s xmms > /dev/null")) > 0) {
+      //printf("Could not execute pidof or xmms not running\n");
+      return FALSE;
+    }
+
+    switch (event->keyval)
+      {
+      case KEY_JOG_DOWN:
+	//printf("xmms --fwd\n");
+	fork_exec("xmms --fwd");
+	break;
+      case KEY_JOG_UP:
+	//printf("xmms --rew\n");
+	fork_exec("xmms --rew");
+	break;
+      case KEY_OK:
+	//printf("xmms --play-pause\n");
+	fork_exec("xmms --play-pause");
+	break;
+      case KEY_CANCEL:
+	//printf("xmms --stop\n");
+	fork_exec("xmms --stop");
+	break;
+      }
+  } else {
+    //printf("unknown event\n");
+  }
+
+  return FALSE;
+}
+
 void 
 load_icons(void)
 {
@@ -706,6 +756,7 @@ static void
 blanker_clicked (GtkWidget *w, GdkEventButton *ev)
 {
   gdk_pointer_ungrab (ev->time);
+  gdk_keyboard_ungrab (ev->time);
 
   gtk_widget_hide (blanker_window);
 
@@ -834,7 +885,12 @@ main( int argc, char *argv[])
    act.sa_handler = reap_children;
    sigaction(SIGCHLD, &act, NULL);
 
+   /* Initialise popup window (starts off hidden) and key event handling */
    popup_init(app);
+   g_signal_connect (G_OBJECT (blanker_window), "key-press-event",
+		     G_CALLBACK (key_event_callback), NULL);
+   g_signal_connect (G_OBJECT (blanker_window), "key-release-event",
+		     G_CALLBACK (key_event_callback), NULL);
 
    /* Reset dpms and backlight to config values (just because we can) */
    set_dpms();
